@@ -17,6 +17,7 @@ from sklearn.preprocessing import StandardScaler  # type: ignore
 from .data_management import (
     DownsamplingIndices,
     FDWaveforms,
+    MassRange,
     PrincipalComponentData,
     Residuals,
     SavableData,
@@ -220,17 +221,13 @@ class Model:
         else:
             self.waveform_generator = waveform_generator
 
-        effective_initial_frequency, effective_srate = expand_frequency_range(
-            initial_frequency_hz, srate_hz, mass_range, Dataset.total_mass
-        )
+        self.mass_range = MassRange(np.array(mass_range))
+        self.initial_frequency_hz = initial_frequency_hz
+        self.srate_hz = srate_hz
+        self.multibanding = multibanding
+        self.parameter_generator_kwargs = parameter_generator_kwargs
 
-        self.dataset = Dataset(
-            effective_initial_frequency,
-            effective_srate,
-            waveform_generator=self.waveform_generator,
-            multibanding=multibanding,
-            parameter_generator_kwargs=parameter_generator_kwargs,
-        )
+        self.dataset = self.make_dataset()
 
         if downsampling_training is None:
             self.downsampling_training: DownsamplingTraining = (
@@ -264,6 +261,25 @@ class Model:
 
         return model
 
+    def make_dataset(self) -> Dataset:
+
+        mass_min, mass_max = self.mass_range.mass_range
+
+        effective_initial_frequency, effective_srate = expand_frequency_range(
+            self.initial_frequency_hz,
+            self.srate_hz,
+            (mass_min, mass_max),
+            Dataset.total_mass,
+        )
+
+        return Dataset(
+            effective_initial_frequency,
+            effective_srate,
+            waveform_generator=self.waveform_generator,
+            multibanding=self.multibanding,
+            parameter_generator_kwargs=self.parameter_generator_kwargs,
+        )
+
     @property
     def auxiliary_data_available(self) -> bool:
         return self.pca_data is not None and self.downsampling_indices is not None
@@ -374,6 +390,7 @@ class Model:
             self.downsampling_indices,
             self.pca_data,
             self.training_parameters,
+            self.mass_range,
         ]
 
         if include_training_data:
@@ -416,7 +433,6 @@ class Model:
         self.downsampling_indices = DownsamplingIndices.from_file(file_arrays)
         self.pca_data = PrincipalComponentData.from_file(file_arrays)
         self.training_parameters = ParameterSet.from_file(file_arrays)
-
         if (
             self.downsampling_indices is None
             or self.pca_data is None
@@ -424,6 +440,12 @@ class Model:
         ):
             raise FileNotFoundError
 
+        mass_range = MassRange.from_file(file_arrays)
+        assert mass_range is not None
+        self.mass_range = mass_range
+
+        self.dataset = self.make_dataset()
+
         self.training_dataset = Residuals.from_file(
             file_arrays, ignore_warnings=ignore_warnings
         )
@@ -650,6 +672,7 @@ class Model:
             self.dataset.frequencies[self.downsampling_indices.phase_indices],
         )
 
+        # downsampled amplitude array
         amp_ds = combine_residuals_amp(residuals.amplitude_residuals[0], pn_amplitude)
         phi_ds = combine_residuals_phi(residuals.phase_residuals[0], pn_phase)
 
@@ -669,82 +692,22 @@ class Model:
             + (2 * np.pi * params.time_shift) * frequencies
         )
 
-        cartesian_waveform = combine_amp_phase(amp, phi)
+        cartesian_waveform_real, cartesian_waveform_imag = combine_amp_phase(amp, phi)
 
         pre = self.dataset.mlgw_bns_prefactor(intrinsic_params.eta, params.total_mass)
         cosi = np.cos(params.inclination)
         pre_plus = (1 + cosi ** 2) / 2 * pre / params.distance_mpc
-        pre_cross = cosi * pre * (-1j) / params.distance_mpc
+        pre_cross = cosi * pre / params.distance_mpc
 
-        return compute_polarizations(cartesian_waveform, pre_plus, pre_cross)
-
-
-class ModelPredictingInverted(Model):
-    def predict(self, frequencies: np.ndarray, params: ParametersWithExtrinsic):
-        """Calculate the waveforms in the plus and cross polarizations,
-        accounting for extrinsic parameters
-
-        Parameters
-        ----------
-        frequencies : np.ndarray
-                Frequencies where to compute the waveform, in Hz.
-        params : ParametersWithExtrinsic
-                Parameters for the waveform, both intrinsic and extrinsic.
-
-        Returns
-        -------
-        hp, hc (complex np.ndarray)
-                Cartesian plus and cross-polarized waveforms, computed
-                at the given frequencies, measured in 1/Hz.
-
-        """
-        assert self.downsampling_indices is not None
-        assert self.nn is not None
-
-        intrinsic_params = params.intrinsic(self.dataset)
-
-        residuals = self.predict_residuals_bulk(
-            ParameterSet.from_list_of_waveform_parameters([intrinsic_params]), self.nn
+        return compute_polarizations(
+            cartesian_waveform_real, cartesian_waveform_imag, pre_plus, pre_cross
         )
 
-        amp_residuals = self.downsampling_training.resample(
-            self.dataset.frequencies_hz[self.downsampling_indices.amplitude_indices],
-            frequencies,
-            residuals.amplitude_residuals[0],
-        )
-        phi_residuals = self.downsampling_training.resample(
-            self.dataset.frequencies_hz[self.downsampling_indices.phase_indices],
-            frequencies,
-            residuals.phase_residuals[0],
-        )
-
-        pn_amplitude = self.dataset.waveform_generator.post_newtonian_amplitude(
-            intrinsic_params, frequencies * params.mass_sum_seconds
-        )
-        pn_phase = self.dataset.waveform_generator.post_newtonian_phase(
-            intrinsic_params, frequencies * params.mass_sum_seconds
-        )
-
-        cartesian_waveform = compute_cartesian_waveform(
-            amp_residuals,
-            pn_amplitude,
-            phi_residuals,
-            pn_phase,
-            params.reference_phase,
-            params.time_shift,
-            frequencies,
-        )
-
-        pre = self.dataset.mlgw_bns_prefactor(intrinsic_params.eta, params.total_mass)
-        cosi = np.cos(params.inclination)
-        pre_plus = (1 + cosi ** 2) / 2 * pre / params.distance_mpc
-        pre_cross = cosi * pre * (-1j) / params.distance_mpc
-
-        return compute_polarizations(cartesian_waveform, pre_plus, pre_cross)
-
 
 @njit
-def combine_amp_phase(amp: np.ndarray, phase: np.ndarray) -> np.ndarray:
+def combine_amp_phase(
+    amp: np.ndarray, phase: np.ndarray
+) -> tuple[np.ndarray, np.ndarray]:
     """Combine amplitude and phase arrays into a Cartesian waveform,
     according to
     :math:`h = A e^{i \phi}`.
@@ -760,7 +723,8 @@ def combine_amp_phase(amp: np.ndarray, phase: np.ndarray) -> np.ndarray:
     -------
     np.ndarray
     """
-    return amp * np.exp(1j * phase)
+    # return amp * np.exp(1j * phase)
+    return (amp * np.cos(phase), amp * np.sin(phase))
 
 
 @njit
@@ -805,7 +769,8 @@ def combine_residuals_phi(phi: np.ndarray, phi_pn: np.ndarray) -> np.ndarray:
 
 @njit
 def compute_polarizations(
-    waveform: np.ndarray,
+    waveform_real: np.ndarray,
+    waveform_imag: np.ndarray,
     pre_plus: Union[complex, float],
     pre_cross: Union[complex, float],
 ) -> tuple[np.ndarray, np.ndarray]:
@@ -819,126 +784,27 @@ def compute_polarizations(
 
     Parameters
     ----------
-    waveform : np.ndarray
-        Cartesian complex-valued waveform.
+    waveform_real : np.ndarray
+        Real part of the cartesian complex-valued waveform.
+    waveform_imag : np.ndarray
+        Imaginary part of the cartesian complex-valued waveform.
     pre_plus : complex
-        Complex-valued prefactor for the plus polarization of the waveform.
+        Real-valued prefactor for the plus polarization of the waveform.
     pre_cross : complex
-        Complex-valued prefactor for the cross polarization of the waveform.
+        Real-valued prefactor for the cross polarization of the waveform.
 
     Returns
     -------
     tuple[np.ndarray, np.ndarray]
         Plus and cross polarizations: complex-valued arrays.
     """
-    hp = pre_plus * waveform
-    hc = pre_cross * waveform
+
+    hp = pre_plus * waveform_real + 1j * pre_plus * waveform_imag
+    hc = pre_cross * waveform_imag - 1j * pre_cross * waveform_real
+
     return hp, hc
 
 
-def cartesian_waveforms_at_frequencies(
-    waveforms: FDWaveforms,
-    new_frequencies: np.ndarray,
-    dataset: Dataset,
-    downsampling_training: DownsamplingTraining,
-    downsampling_indices: DownsamplingIndices,
-) -> np.ndarray:
-    """Starting from an array of downsampled waveforms decomposed into
-    amplitude and phase, interpolate them to a new frequency grid and
-    put them in Cartesian form.
-
-    Parameters
-    ----------
-    waveforms : FDWaveforms
-        Waveforms to put in Cartesian form.
-    new_frequencies : np.ndarray
-        Frequencies to resample to, in natural units.
-    dataset : Dataset
-        Reference dataset.
-    downsampling_training : DownsamplingTraining
-        Training model for the downsampling, contains metadata needed for the resampling.
-    downsampling_indices : DownsamplingIndices
-        Downsampling indices, needed for the resampling.
-
-    Returns
-    -------
-    np.ndarray
-        Cartesian waveforms: complex array with shape
-        ``(n_waveforms, len(new_frequencies))``.
-    """
-
-    amp_frequencies = dataset.frequencies[downsampling_indices.amplitude_indices]
-    phi_frequencies = dataset.frequencies[downsampling_indices.phase_indices]
-
-    amps = np.array(
-        [
-            downsampling_training.resample(amp_frequencies, new_frequencies, amp)
-            for amp in waveforms.amplitudes
-        ]
-    )
-
-    phis = np.array(
-        [
-            downsampling_training.resample(phi_frequencies, new_frequencies, phi)
-            for phi in waveforms.phases
-        ]
-    )
-
-    return amps * np.exp(1j * phis)
-
-
-@njit
-def compute_cartesian_waveform(
-    amplitude_residuals: np.ndarray,
-    amp_pn: np.ndarray,
-    phi_residuals: np.ndarray,
-    phi_pn: np.ndarray,
-    reference_phase: float,
-    time_shift: float,
-    frequencies: np.ndarray,
-) -> np.ndarray:
-    r"""Compute the Cartesian form of the waveform, starting
-    from the residuals, the post-Newtonian baseline,
-    and the parameters to add a linear term to the phase.
-
-    This function is separated out so that it can be
-    decorated with :func:`numba.njit`.
-
-    Parameters
-    ----------
-    amplitude_residuals : np.ndarray
-        Amplitude residuals, in the form :math:`\log (A / A_{\text{PN}})`.
-    amp_pn : np.ndarray
-        Post-Newtonian baseline amplitude.
-    phi_residuals : np.ndarray
-        Phase residuals.
-    phi_pn : np.ndarray
-        Post-Newtonian baseline phase
-    reference_phase : float
-        Overall phase to add.
-    time_shift : float
-        Time-domain shift in seconds, corresponds to
-        a linear term added to the phase.
-    frequencies : np.ndarray
-        Reference frequencies, in Hz.
-
-    Returns
-    -------
-    np.ndarray
-        Cartesian waveform, :math:`h = A e^{i \phi}`.
-    """
-
-    amp = np.exp(amplitude_residuals) * amp_pn
-    phi = (
-        phi_residuals
-        + phi_pn
-        + reference_phase
-        + (2 * np.pi * time_shift) * frequencies
-    )
-
-    return amp * np.exp(1j * phi)
-
-
 def expand_frequency_range(
     initial_frequency: float,
     final_frequency: float,
